@@ -13,6 +13,12 @@
  */
 
 #include <config.h>
+
+#include <ctype.h>
+#include <math.h>
+#include <stdio.h>
+
+#include <gdk/gdkx.h> /* for BlackPixel */
 #include <gtk/gtkenums.h>
 #include <gtk/gtkentry.h>
 #include <gtk/gtkwindow.h>
@@ -20,16 +26,12 @@
 #include <gtk/gtksignal.h>
 #include <gdk/gdkkeysyms.h>
 #include <libgnomeui/gnome-canvas.h>
-#include <stdio.h>
-#include "e-table-sorted-variable.h"
+
+#include "e-tree-table-adapter.h"
 #include "e-tree-model.h"
-#include "e-cell-tree.h"
 #include "gal/util/e-util.h"
 #include "e-table-item.h"
-
-#include <gdk/gdkx.h> /* for BlackPixel */
-#include <ctype.h>
-#include <math.h>
+#include "e-cell-tree.h"
 
 #include "tree-expanded.xpm"
 #include "tree-unexpanded.xpm"
@@ -49,29 +51,46 @@ static ECellClass *parent_class;
 
 #define INDENT_AMOUNT 16
 
-static int
-visible_depth_of_node (ETreeModel *tree_model, ETreePath *path)
-{
-	return (e_tree_model_node_depth (tree_model, path) 
-		- (e_tree_model_root_node_is_visible (tree_model) ? 0 : 1));
-}
-
-static gint
-offset_of_node (ETreeModel *tree_model, ETreePath *path)
-{
-	return (visible_depth_of_node(tree_model, path) + 1) * INDENT_AMOUNT;
-}
-
-static ETreePath*
+static ETreePath
 e_cell_tree_get_node (ETableModel *table_model, int row)
 {
-	return (ETreePath*)e_table_model_value_at (table_model, -1, row);
+	return e_table_model_value_at (table_model, -1, row);
 }
 
 static ETreeModel*
 e_cell_tree_get_tree_model (ETableModel *table_model, int row)
 {
-	return (ETreeModel*)e_table_model_value_at (table_model, -2, row);
+	return e_table_model_value_at (table_model, -2, row);
+}
+
+static ETreeTableAdapter *
+e_cell_tree_get_tree_table_adapter (ETableModel *table_model, int row)
+{
+	return e_table_model_value_at (table_model, -3, row);
+}
+
+static int
+visible_depth_of_node (ETableModel *model, int row)
+{
+	ETreeModel *tree_model = e_cell_tree_get_tree_model(model, row);
+	ETreeTableAdapter *adapter = e_cell_tree_get_tree_table_adapter(model, row);
+	ETreePath path = e_cell_tree_get_node(model, row);
+	return (e_tree_model_node_depth (tree_model, path)
+		- (e_tree_table_adapter_root_node_is_visible (adapter) ? 0 : 1));
+}
+
+static gint
+offset_of_node (ETableModel *table_model, int row)
+{
+	ETreeModel *tree_model = e_cell_tree_get_tree_model(table_model, row);
+	ETreePath path = e_cell_tree_get_node(table_model, row);
+
+	if (visible_depth_of_node (table_model, row) > 0 ||
+	    e_tree_model_node_is_expandable(tree_model, path)) {
+		return (visible_depth_of_node(table_model, row) + 1) * INDENT_AMOUNT;
+	} else {
+		return 0;
+	}
 }
 
 /*
@@ -159,7 +178,8 @@ ect_draw (ECellView *ecell_view, GdkDrawable *drawable,
 {
 	ECellTreeView *tree_view = (ECellTreeView *)ecell_view;
 	ETreeModel *tree_model = e_cell_tree_get_tree_model(ecell_view->e_table_model, row);
-	ETreePath *node;
+	ETreeTableAdapter *tree_table_adapter = e_cell_tree_get_tree_table_adapter(ecell_view->e_table_model, row);
+	ETreePath node;
 	GdkRectangle rect, *clip_rect;
 	GtkWidget *canvas = GTK_WIDGET (tree_view->canvas);
 	GdkGC *fg_gc = canvas->style->fg_gc[GTK_STATE_ACTIVE];
@@ -167,7 +187,6 @@ ect_draw (ECellView *ecell_view, GdkDrawable *drawable,
 	gboolean selected;
 
 	int offset, subcell_offset;
-	gboolean expanded, expandable;
 
 	selected = flags & E_CELL_SELECTED;
 
@@ -175,16 +194,14 @@ ect_draw (ECellView *ecell_view, GdkDrawable *drawable,
 	if (/* XXX */ TRUE) {
 		GdkPixbuf *node_image;
 		int node_image_width = 0, node_image_height = 0;
-		ETreePath *parent_node;
+		ETreePath parent_node;
 
 		node = e_cell_tree_get_node (ecell_view->e_table_model, row);
 
-		offset = offset_of_node (tree_model, node);
-		expandable = e_tree_model_node_is_expandable (tree_model, node);
-		expanded = e_tree_model_node_is_expanded (tree_model, node);
+		offset = offset_of_node (ecell_view->e_table_model, row);
 		subcell_offset = offset;
 
-		node_image = e_tree_model_icon_of_node (tree_model, node);
+		node_image = e_tree_model_icon_at (tree_model, node);
 
 		if (node_image) {
 			node_image_width = gdk_pixbuf_get_width (node_image);
@@ -214,7 +231,9 @@ ect_draw (ECellView *ecell_view, GdkDrawable *drawable,
 		/* draw our lines */
 		if (E_CELL_TREE(tree_view->cell_view.ecell)->draw_lines) {
 
-			if (visible_depth_of_node (tree_model, node) > 0
+			int depth;
+
+			if (visible_depth_of_node (ecell_view->e_table_model, row) > 0
 			    || e_tree_model_node_get_children (tree_model, node, NULL) > 0)
 				gdk_draw_line (drawable, tree_view->gc,
 					       rect.x + offset - INDENT_AMOUNT / 2 + 1,
@@ -222,7 +241,7 @@ ect_draw (ECellView *ecell_view, GdkDrawable *drawable,
 					       rect.x + offset,
 					       rect.y + rect.height / 2);
 
-			if (visible_depth_of_node (tree_model, node) != 0) {
+			if (visible_depth_of_node (ecell_view->e_table_model, row) != 0) {
 				gdk_draw_line (drawable, tree_view->gc,
 					       rect.x + offset - INDENT_AMOUNT / 2,
 					       rect.y,
@@ -237,7 +256,8 @@ ect_draw (ECellView *ecell_view, GdkDrawable *drawable,
 			   correct vertical pipe for it's configuration. */
 			parent_node = e_tree_model_node_get_parent (tree_model, node);
 			offset -= INDENT_AMOUNT;
-			while (parent_node && visible_depth_of_node (tree_model, parent_node) != 0) {
+			depth = visible_depth_of_node (ecell_view->e_table_model, row) - 1;
+			while (parent_node && depth != 0) {
 				if (e_tree_model_node_get_next(tree_model, parent_node)) {
 					gdk_draw_line (drawable, tree_view->gc,
 						       rect.x + offset - INDENT_AMOUNT / 2,
@@ -246,16 +266,17 @@ ect_draw (ECellView *ecell_view, GdkDrawable *drawable,
 						       rect.y + rect.height);
 				}
 				parent_node = e_tree_model_node_get_parent (tree_model, parent_node);
+				depth --;
 				offset -= INDENT_AMOUNT;
 			}
 		}
 
 		/* now draw our icon if we're expandable */
-		if (expandable) {
+		if (e_tree_model_node_is_expandable (tree_model, node)) {
 			GdkPixbuf *image;
 			int image_width, image_height;
 
-			image = (expanded 
+			image = (e_tree_table_adapter_node_is_expanded (tree_table_adapter, node)
 				 ? E_CELL_TREE(tree_view->cell_view.ecell)->open_pixbuf
 				 : E_CELL_TREE(tree_view->cell_view.ecell)->closed_pixbuf);
 
@@ -304,8 +325,9 @@ ect_event (ECellView *ecell_view, GdkEvent *event, int model_col, int view_col,
 {
 	ECellTreeView *tree_view = (ECellTreeView *) ecell_view;
 	ETreeModel *tree_model = e_cell_tree_get_tree_model (ecell_view->e_table_model, row);
-	ETreePath *node = e_cell_tree_get_node (ecell_view->e_table_model, row);
-	int offset = offset_of_node (tree_model, node);
+	ETreeTableAdapter *tree_table_adapter = e_cell_tree_get_tree_table_adapter(ecell_view->e_table_model, row);
+	ETreePath node = e_cell_tree_get_node (ecell_view->e_table_model, row);
+	int offset = offset_of_node (ecell_view->e_table_model, row);
 
 	switch (event->type) {
 	case GDK_BUTTON_PRESS: {
@@ -315,9 +337,9 @@ ect_event (ECellView *ecell_view, GdkEvent *event, int model_col, int view_col,
 		/* only activate the tree control if the click/release happens in the icon's area. */
 		if (event->button.x > (offset - INDENT_AMOUNT) && event->button.x < offset) {
 			if (e_tree_model_node_is_expandable (tree_model, node)) {
-				e_tree_model_node_set_expanded (tree_model,
-								node,
-								!e_tree_model_node_is_expanded(tree_model, node));
+				e_tree_table_adapter_node_set_expanded (tree_table_adapter,
+									node,
+									!e_tree_table_adapter_node_is_expanded(tree_table_adapter, node));
 				return TRUE;
 			}
 		}
@@ -340,6 +362,7 @@ ect_event (ECellView *ecell_view, GdkEvent *event, int model_col, int view_col,
 			break;
 		default:
 			/* nada */
+			break;
 		}
 
 		return_value = e_cell_event(tree_view->subcell_view, event, model_col, view_col, row, flags, actions);
@@ -357,6 +380,7 @@ ect_event (ECellView *ecell_view, GdkEvent *event, int model_col, int view_col,
 			break;
 		default:
 			/* nada */
+			break;
 		}
 
 		return return_value;
@@ -386,12 +410,16 @@ ect_max_width (ECellView *ecell_view, int model_col, int view_col)
 	int number_of_rows;
 	int max_width = 0;
 	int width = 0;
+	int subcell_max_width;
 
 	number_of_rows = e_table_model_row_count (ecell_view->e_table_model);
+
+	subcell_max_width = e_cell_max_width (tree_view->subcell_view, model_col, view_col);
 	
 	for (row = 0; row < number_of_rows; row++) {
 		ETreeModel *tree_model = e_cell_tree_get_tree_model(ecell_view->e_table_model, row);
-		ETreePath *node;
+		ETreeTableAdapter *tree_table_adapter = e_cell_tree_get_tree_table_adapter(ecell_view->e_table_model, row);
+		ETreePath node;
 		GdkPixbuf *node_image;
 		int node_image_width = 0, node_image_height = 0;
 		
@@ -400,19 +428,19 @@ ect_max_width (ECellView *ecell_view, int model_col, int view_col)
 		
 		node = e_cell_tree_get_node (ecell_view->e_table_model, row);
 		
-		offset = offset_of_node (tree_model, node);
+		offset = offset_of_node (ecell_view->e_table_model, row);
 		expandable = e_tree_model_node_is_expandable (tree_model, node);
-		expanded = e_tree_model_node_is_expanded (tree_model, node);
+		expanded = e_tree_table_adapter_node_is_expanded (tree_table_adapter, node);
 		subcell_offset = offset;
 
-		node_image = e_tree_model_icon_of_node (tree_model, node);
+		node_image = e_tree_model_icon_at (tree_model, node);
 
 		if (node_image) {
 			node_image_width = gdk_pixbuf_get_width (node_image);
 			node_image_height = gdk_pixbuf_get_height (node_image);
 		}
 
-		width = subcell_offset + node_image_width;
+		width = subcell_max_width + subcell_offset + node_image_width;
 
 		if (expandable) {
 			GdkPixbuf *image;
@@ -424,9 +452,6 @@ ect_max_width (ECellView *ecell_view, int model_col, int view_col)
 			width += gdk_pixbuf_get_width(image);
 		}
 
-		width += e_cell_max_width (tree_view->subcell_view, model_col, 
-					   view_col);
-
 		max_width = MAX (max_width, width);
 	}
 
@@ -442,11 +467,11 @@ ect_show_tooltip (ECellView *ecell_view, int model_col, int view_col, int row,
 {		
 	ECellTreeView *tree_view = (ECellTreeView *) ecell_view;
 	ETreeModel *tree_model = e_cell_tree_get_tree_model (ecell_view->e_table_model, row);
-	ETreePath *node = e_cell_tree_get_node (ecell_view->e_table_model, row);
-	int offset = offset_of_node (tree_model, node);
+	ETreePath node = e_cell_tree_get_node (ecell_view->e_table_model, row);
+	int offset = offset_of_node (ecell_view->e_table_model, row);
 	GdkPixbuf *node_image;
 
-	node_image = e_tree_model_icon_of_node (tree_model, node);
+	node_image = e_tree_model_icon_at (tree_model, node);
 	if (node_image)
 		offset += gdk_pixbuf_get_width (node_image);
 
@@ -487,14 +512,16 @@ ect_print (ECellView *ecell_view, GnomePrintContext *context,
 
 	if (/* XXX only if we're the active sort */ TRUE) {
 		ETreeModel *tree_model = e_cell_tree_get_tree_model (ecell_view->e_table_model, row);
-		ETreePath *node = e_cell_tree_get_node (ecell_view->e_table_model, row);
-		int offset = offset_of_node (tree_model, node);
+		ETreeTableAdapter *tree_table_adapter = e_cell_tree_get_tree_table_adapter(ecell_view->e_table_model, row);
+		ETreePath node = e_cell_tree_get_node (ecell_view->e_table_model, row);
+		int offset = offset_of_node (ecell_view->e_table_model, row);
 		int subcell_offset = offset;
 		gboolean expandable = e_tree_model_node_is_expandable (tree_model, node);
-		gboolean expanded = e_tree_model_node_is_expanded (tree_model, node);
+		gboolean expanded = e_tree_table_adapter_node_is_expanded (tree_table_adapter, node);
 
 		/* draw our lines */
 		if (E_CELL_TREE(tree_view->cell_view.ecell)->draw_lines) {
+			int depth;
 
 			if (!e_tree_model_node_is_root (tree_model, node)
 			    || e_tree_model_node_get_children (tree_model, node, NULL) > 0) {
@@ -507,7 +534,7 @@ ect_print (ECellView *ecell_view, GnomePrintContext *context,
 						    height / 2);
 			}
 
-			if (visible_depth_of_node (tree_model, node) != 0) {
+			if (visible_depth_of_node (ecell_view->e_table_model, row) != 0) {
 				gnome_print_moveto (context,
 						    offset - INDENT_AMOUNT / 2,
 						    height);
@@ -522,8 +549,9 @@ ect_print (ECellView *ecell_view, GnomePrintContext *context,
 			   each level if the node has siblings, and drawing the
 			   correct vertical pipe for it's configuration. */
 			node = e_tree_model_node_get_parent (tree_model, node);
+			depth = visible_depth_of_node (ecell_view->e_table_model, row) - 1;
 			offset -= INDENT_AMOUNT;
-			while (node && visible_depth_of_node (tree_model, node) != 0) {
+			while (node && depth != 0) {
 				if (e_tree_model_node_get_next(tree_model, node)) {
 					gnome_print_moveto (context,
 							    offset - INDENT_AMOUNT / 2,
@@ -533,6 +561,7 @@ ect_print (ECellView *ecell_view, GnomePrintContext *context,
 							    0);
 				}
 				node = e_tree_model_node_get_parent (tree_model, node);
+				depth --;
 				offset -= INDENT_AMOUNT;
 			}
 		}
