@@ -20,13 +20,14 @@ typedef struct _ColorCombo {
 	ColorPalette    *palette;
 
         GdkColor *default_color;
+	gboolean  trigger;
 } ColorCombo;
 
 typedef struct {
 	GtkComboBoxClass parent_class;
 
 	/* Signals emited by this widget */
-	void (* changed) (ColorCombo *color_combo, GdkColor *color);
+	void (* changed) (ColorCombo *color_combo, GdkColor *color, gboolean by_user);
 } ColorComboClass;
 
 #define COLOR_COMBO_TYPE     (color_combo_get_type ())
@@ -39,9 +40,10 @@ GtkType    color_combo_get_type   (void);
 GtkWidget *color_combo_new        (char       **icon,
 				   const char  *no_color_label,
 				   GdkColor    *default_color,
-				   gchar       *group_name);
+				   ColorGroup  *color_group);
 void       color_combo_set_color  (ColorCombo  *cc,
 				   GdkColor    *color);
+GdkColor  *color_combo_get_color  (ColorCombo  *cc);
 
 END_GNOME_DECLS
 
